# exlap_c.rb  version 0.8d: (coding: utf-8)  2010/10/17
  # for ruby ver 1.9.x
  # Copyright (C) T. Yoshiizumi, 2010 All rights reserved.

$BASE_C = 'w'  # utf-8を基本の文字コードにする(nkfのオプション文字)
$CSV_C = 's'

# part 1:  「配列を材料にxmlssを書き出す」ためのメソッドなど
require "nkf"
require "rexml/document"

Outset_workbook = <<EOS1
doc = REXML::Document.new(\"<?xml version='1.0' encoding='UTF-8'?>\\n<?mso-application progid=\\\"Excel.Sheet\\\"?>\\n\")
workbook = doc.add_element(\"Workbook\", {\"xmlns:ss\"=>\"urn:schemas-microsoft-com:office:spreadsheet\",
  \"xmlns:html\"=>\"http://www.w3.org/TR/REC-html40\",
  \"xmlns:x\"=>\"urn:schemas-microsoft-com:office:excel\",
  \"xmlns:o\"=>\"urn:schemas-microsoft-com:office:office\",
  \"xmlns\"=>\"urn:schemas-microsoft-com:office:spreadsheet\"}).add_text(\"\\n \")
document_properties = workbook.add_element(\"DocumentProperties\", {\"xmlns\"=>\"urn:schemas-microsoft-com:office:office\"}).add_text(\"\\n  \")
author = document_properties.add_element(\"Author\")
author.add_text(\"__author__\")
last_author = document_properties.add_element(\"LastAuthor\")
last_author.add_text(\"__last_author__\")
styles = workbook.add_element(\"Styles\").add_text(\"\\n  \")
style = styles.add_element(\"Style\", {\"ss:Name\"=>\"Normal\",
  \"ss:ID\"=>\"Default\"}).add_text(\"\\n   \")
alignment = style.add_element(\"Alignment\", {\"ss:Vertical\"=>\"Center\"})
borders = style.add_element(\"Borders\")
font = style.add_element(\"Font\", {\"ss:Size\"=>\"11\",
  \"x:CharSet\"=>\"128\",
  \"ss:Color\"=>\"#000000\",
  \"ss:FontName\"=>\"ＭＳ Ｐゴシック\",
  \"x:Family\"=>\"Modern\"})
interior = style.add_element(\"Interior\")
number_format = style.add_element(\"NumberFormat\")
protection = style.add_element(\"Protection\")
EOS1

Outset_worksheet = <<EOS2
worksheet = workbook.add_element(\"Worksheet\", {\"ss:Name\"=>\"__sheet_name__\"}).add_text(\"\\n  \")
table = worksheet.add_element(\"Table\", {
  \"ss:DefaultRowHeight\"=>\"13.5\",
  \"ss:DefaultColumnWidth\"=>\"54\",
  \"ss:ExpandedRowCount\"=>\"__max_row__\",
  \"ss:ExpandedColumnCount\"=>\"__max_column__\",
  \"x:FullColumns\"=>\"1\",
  \"x:FullRows\"=>\"1\"}).add_text(\"\\n   \")
EOS2

MaxDay = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]  # 各月の最終日
Wareki = {'明治'=>1867, '大正'=>1911, '昭和'=>1925, '平成'=>1988}

# 数値型関係の文字を全角から半角に変換
def zen_to_han(str)
  str = str.gsub(/０/, "0")
  str.gsub!(/１/, "1")
  str.gsub!(/２/, "2")
  str.gsub!(/３/, "3")
  str.gsub!(/４/, "4")
  str.gsub!(/５/, "5")
  str.gsub!(/６/, "6")
  str.gsub!(/７/, "7")
  str.gsub!(/８/, "8")
  str.gsub!(/９/, "9")
  str.gsub!(/，/, ",")
  str.gsub!(/．/, ".")
  str.gsub!(/％/, "%")
  str.gsub!(/／/, "/")
  str.gsub!(/−/, "-")
  str.gsub!(/：/, ":")
  return str
end

# コントロールコード部分の調整。例：  \t → {[{9}]} → &#9;
  # いきなり半角 '&' を使うと &amp; に変換されてしまうので２段階方式
def arrange_char(s, opt)
  if opt == :STEP1
    s = s.gsub(/\r\n/, "\n")
    s.gsub!(/\n/, "\r\n")
    if RUBY_VERSION >= "1.9"
      s.gsub!(/[\x01-\x1f]/) {|mp|
        mp = "{[{#{mp.getbyte(0)}}]}"
      }
    else  # if RUBY_VERSION < "1.9"
      s.gsub!(/[\x01-\x1f]/) {|mp|
        mp = "{[{#{mp[0]}}]}"
      }
    end
  elsif opt == :STEP2
    s = s.gsub(/\{\[\{(\d+)\}\]\}/, "&#\\1;")
    s = s.gsub(/\{\[\{([a-z]+)\}\]\}/i, "&\\1;")
    s = s.gsub(/\343\200\234/, "&#65374;")  # utf-8の「〜」
    s = s.gsub(/\342\210\222/, "&#65293;")  # utf-8の「−」
  end
  return s
end

# Hashをrubyスクリプトの形式に変換
def hash_to_str(hs)
  hs_str = "{"
  hs.each {|key,val|
    hs_str += sprintf("%s=>%s, ", key.dump, val.dump)
  }
  hs_str = hs_str.sub(/, $/, "}")
  hs_str = hs_str.gsub(/(\\\d{3})+/) {|mp|
    mp = eval("\"#{mp}\"")
  }
  return hs_str
end

# 数値型の時の書式を得る
def get_number_format(data_str)
  fmt = nil
  ii = ff = nil  # ii:整数部, ff:小数部
  if data_str =~ /^-?(\d[0-9,]*)\.(\d+)$/  # 小数点数
    ii = $1
    ff = $2
  elsif data_str =~ /^-?(\d[0-9,]*)$/  # 整数
    ii = $1
  end
  if ii
    if ii =~ /^0/
      fmt = ii.gsub(/\d/, "0")
    else
      fmt = ii.gsub(/\d/, "#")
      fmt = fmt.sub(/^-/, "")
      fmt = fmt.sub(/\#$/, "0")
      fmt = fmt.sub(/^\#+0$/, "0")
    end
    if fmt =~ /^[0#,]+([0#],[0#]{3})$/
      fmt = $1
    end
    if ff
      fmt = fmt + '.' + ff.gsub(/\d/, "0")
    end
  end
  return fmt
end

# styleを扱うためのクラス。現状ではNumberFormatのみに着目。
class StyleScript
  def initialize
    @record = Array.new  # NumberFormatをこれに記録
    @script = ''  # styleに関するスrexml scriptをこれに記録
  end

  def get_id(number_format)
    n = @record.index(number_format)
    unless n
      @record << number_format
      n = @record.index(number_format)
      @script += %Q!style = styles.add_element("Style", {"ss:ID"=>"sid#{n}"}).add_text("\\n   ")\n!
      @script += %Q!number_format = style.add_element("NumberFormat", {"ss:Format"=>"#{number_format}"})\n!
    end
    return n
  end

  def get_script
    return @script
  end
end

# xmlssの中のcellを扱うためのクラス。<Cell>……</Cell>の書き出しが目的
class CellScript
  def initialize()
    @data = @index = nil
    @type = @number_format = @rspan = @cspan = @formula = nil
  end
  attr_accessor :data
  attr_accessor :index
  attr_accessor :type
  attr_accessor :number_format
  attr_accessor :rspan
  attr_accessor :cspan
  attr_accessor :formula

  def set(data, index)
    @index = index
    @type = @number_format = @rspan = @cspan = @formula = nil
    if data.class == Hash
      @data = data['data']
      @type = data['type']
      @number_format = data['number_format']
      @rspan = data['rspan']
      @cspan = data['cspan']
      @formula = data['formula']
    else
      @data = data
    end
    @data = arrange_char(@data, :STEP1)
  end

  def auto_type()
    if !@data or @type == "String"
      return
    end
    data_str = zen_to_han(@data.to_s)
    if data_str =~ /^-?\d[0-9,]*$|^-?\d[0-9,]*\.\d+$/  # Number
      @data = data_str.gsub(/,/, "").sub(/^(-?)0+/, "\\1")
      @type = "Number"
      fmt = get_number_format(data_str)
      if !@number_format and fmt
        @number_format = fmt
      end
    elsif data_str =~ /^-?\d[0-9,.]*\%$/  # Percent
      if (fmt = get_number_format(data_str.sub(/\%$/, ""))) != nil
        val = data_str.gsub(/[,%]/, "")
        val = val.to_f / 100.0
        @data = val.to_s
        @type = "Number"
        @number_format = fmt + '%'
      end
    elsif data_str =~ /^(\d{4})[\-\/](\d\d?)[\-\/](\d\d?)$/ or  # Date
          data_str =~ /^(\d{4})年(\d\d?)月(\d\d?)日$/
      y = $1.to_i;  m = $2.to_i;  d = $3.to_i
      if y >= 1900 and m >= 1 and m <= 12 and d >= 1 and d <= MaxDay[m]
        @data = sprintf("%d-%02d-%02dT00:00:00.000", y, m, d)
        @type = "DateTime"
        @number_format = (data_str =~ /月/) ? "Long Date" : "Short Date"
      end
    elsif data_str =~ /^(明治|大正|昭和|平成)(\d\d?)年(\d\d?)月(\d\d?)日$/
      gengou = $1
      y = $2.to_i;  m = $3.to_i;  d = $4.to_i
      y += Wareki[gengou]
      if y >= 1868 and m >= 1 and m <= 12 and d >= 1 and d <= MaxDay[m]
        @data = sprintf("%d-%02d-%02dT00:00:00.000", y, m, d)
        @type = "DateTime"
        @number_format = "[JPN][$-411]gggy{[{quot}]}年{[{quot}]}m{[{quot}]}月{[{quot}]}d{[{quot}]}日{[{quot}]}"
      end
    elsif data_str =~ /^(\d+)\:(\d+)(\:\d+)?$/
      hh = $1.to_i;  mm = $2.to_i;  ss = $3
      if ss
        ss = ss.sub(/^\:/, "")
        ss = ss.to_i
        @type = "DateTime"
        @number_format = "h:mm:ss"
        @data = sprintf("1899-12-31T%02d:%02d:%02d.000", hh, mm, ss)
      else
        @type = "DateTime"
        @number_format = "Short Time"
        @data = sprintf("1899-12-31T%02d:%02d:00.000", hh, mm)
      end
    end
    if !@type
      @type = "String"
    end
  end

  def get_script()
    script_str = ''
    if !@data or !@index
      return script_str
    end
    hs = Hash.new
    hs['ss:Index'] = @index.to_s
    if @rspan and @rspan.to_i > 1
      hs['ss:MergeDown'] = (@rspan.to_i - 1).to_s
    end
    if @cspan and @cspan.to_i > 1
      hs['ss:MergeAcross'] = (@cspan.to_i - 1).to_s
    end
    if @formula
      hs['ss:Formula'] = @formula
    end
    if @number_format
      n = $style_info.get_id(@number_format)
      hs['ss:StyleID'] = "sid#{n}"
    end
    script_str += %Q!cell = row.add_element("Cell", #{hash_to_str(hs)})\n!
    script_str += %Q!data = cell.add_element("Data", {"ss:Type"=>"#{@type}"})\n!
    script_str += %Q!data.add_text(#{@data.dump})\n!
    script_str.gsub!(/(\\\d{3})+/) {|mp|
      mp = eval("\"#{mp}\"")
    }
    return script_str
  end
end


# 配列の各要素の調整
  # 各セルについて、$my_convertをevalで実行し、変換を施す
  # 第２引数に :TO_STRING を指定すれば、Hashの要素をStringにする。
  # 文字コードをbase_codeに合わせる
def arrange_array(ary, form=nil, base_code=nil)
  aa = ary.dup
  base_code = $BASE_C  unless base_code
  if !$my_convert and form != :TO_STRING
    jp_charset = [NKF::EUC, NKF::JIS, NKF::SJIS, NKF::UTF8]
    code = nil
    for rn in 0...aa.size  # 配列の文字コードを調べる
      next  unless aa[rn]
      for cn in 0...aa[rn].size
        next  unless aa[rn][cn]
        if aa[rn][cn].class == Hash
          if aa[rn][cn]['data'] and aa[rn][cn]['data'].class == String
            wcode = NKF::guess(aa[rn][cn]['data'])
            if jp_charset.include?(wcode)
              code = wcode
              break
            end
          end
        else
          if aa[rn][cn] and aa[rn][cn].class == String
            wcode = NKF::guess(aa[rn][cn])
            if jp_charset.include?(wcode)
              code = wcode
              break
            end
          end
        end
      end
      break  if code
    end
    if code == nil or  # 文字コード変換の必要はない
       (base_code == 'e' and code == NKF::EUC) or
       (base_code == 'j' and code == NKF::JIS) or
       (base_code == 's' and code == NKF::SJIS) or
       (base_code == 'w' and code == NKF::UTF8)
      return aa
    end
  end
  my_convert = nil
  if $my_convert
    my_convert = NKF.nkf("-m0#{base_code}", $my_convert)
  end
  for rn in 0...aa.size
    next  unless aa[rn]
    for cn in 0...aa[rn].size
      next  unless aa[rn][cn]
      if aa[rn][cn].class == Hash
        if aa[rn][cn]['data']
          str = NKF.nkf("-m0#{base_code}", aa[rn][cn]['data'].to_s)
          eval(my_convert)  if my_convert
          if form != :TO_STRING
            aa[rn][cn]['data'] = str
          else
            aa[rn][cn] = str
          end
        end
      else
        if aa[rn][cn]
          str = NKF.nkf("-m0#{base_code}", aa[rn][cn].to_s)
          eval(my_convert)  if my_convert
          aa[rn][cn] = str
        end
      end
    end
  end
  return aa
end


# xmlssの１つのワークシートを書き出すメソッド
def make_sheet_script(aa, sheet_name, start_rc=nil)
  aa = arrange_array(aa)
  sheet_name = NKF.nkf("-m0#{$BASE_C}", sheet_name.to_s)
  if start_rc and start_rc.class == Array and start_rc.size == 2
    start_r = start_rc[0].to_i
    start_r -= 1  if start_r > 0
    start_c = start_rc[1].to_i
    start_c -= 1  if start_c > 0
  else
    start_r = start_c = 0
  end
  max_row = aa.size
  max_col = 0
  cs = CellScript.new
  script_str = ''
  for rn in 0...max_row
    next  if !aa[rn] or aa[rn].size < 1
    script_str += %Q!row = table.add_element("Row", {"ss:Index"=>"#{rn+1+start_r}"}).add_text("\\n    ")\n!
    max_col = aa[rn].size  if aa[rn].size > max_col
    for cn in 0...aa[rn].size
      next  unless aa[rn][cn]
      data = aa[rn][cn]
      if data.class != Hash
        data = data.to_s
      else
        data['data'] = data['data'].to_s
      end
      next  if data == ''
      cs.set(data, cn+1+start_c)
      cs.auto_type
      script_str += cs.get_script
    end
  end
  outset = Outset_worksheet.dup
  outset.sub!(/__sheet_name__/, sheet_name)
  outset.sub!(/__max_row__/, (max_row + start_r).to_s)
  outset.sub!(/__max_column__/, (max_col + start_c).to_s)
  return outset + script_str
end

# rexml scriptをevalで実行。その結果たるxmlソースを返す
def execute_script(zz_scr)
  xml_str = ''
  zz_scr += "xml_str = doc.to_s\n"
  zz_scr = NKF.nkf("-m0w", zz_scr)
  eval(zz_scr)
  xml_str = arrange_char(xml_str, :STEP2)
  return xml_str
end

# データが格納されている配列を基にして xmlssソース(文字列)を生成して返す。
def make_xml(*param)
  scr = ''
  count = 0
  $style_info = StyleScript.new
  while aa = param.shift
    next  if aa.class != Array
    next  if aa[0].class != Array
    count += 1
    sheet_name = sprintf("Sheet%d", count)
    start_rc = nil
    while param.size > 0
      if param[0].class == String
        sheet_name = param.shift
      elsif param[0].class == Array and param[0][0].class != Array
        start_rc = param.shift
      else
        break
      end
    end
    scr += make_sheet_script(aa, sheet_name, start_rc)
  end
  scr = Outset_workbook + $style_info.get_script + scr
  $style_info = nil
  return execute_script(scr)
end


# part 2:  「csv | web table のデータをxmlssに書き出す」ためのメソッドなど
require "csv"

if (defined?(ENV['OS']) and ENV['OS'] =~ /^Windows/i) and
   (defined?(ENV['HOME']) and ENV['HOME'] =~ /^[A-Z]\:/i)
  $exlap_os = :WIN32
else
  $exlap_os = :COMMON
end


# Webからデータを取得するためのクラス
  # MS-Windowsの場合はMsxml2.xmlhttpを利用。他はopen-uriを利用
class Mshttp
  def initialize(os_val = :COMMON)
    @os_val = os_val
    @MSHTTP = nil
    if @os_val == :WIN32
      require 'win32ole'
      begin
        @MSHTTP = WIN32OLE.new('Msxml2.XMLHTTP')
      rescue
        @MSHTTP = nil
      end
      unless defined? @MSHTTP
        begin
          @MSHTTP = WIN32OLE.new('MSXML.XMLHTTPRequest')
        rescue
          @MSHTTP = nil
        end
      end
      unless defined? @MSHTTP
        begin
          @MSHTTP = WIN32OLE.new('Microsoft.XMLHTTP')
        rescue
          @MSHTTP = nil
        end
      end
      @sec_limit = 20  # タイムアウト秒数
    else
      require 'open-uri'
    end
    @body = ''
  end
  attr_accessor  :sec_limit
  attr_reader  :body

  # htmlドキュメントを取得。失敗時は false または '' を返す
  def web_open(url)
    if @os_val == :WIN32
      unless @MSHTTP
        return false
      end
      if url =~ /^http/i
        @MSHTTP.Open "GET", url, false
        sec_count = 0
        while @MSHTTP.readyState != 1
          sleep 1
          sec_count += 1
          break  if sec_count > @sec_limit
        end
        if sec_count > @sec_limit
          @MSHTTP.abort
          return false
        end
        begin
          @MSHTTP.Send
        rescue
          @MSHTTP.abort
          return false
        end
        sec_count = 0
        while @MSHTTP.readyState != 4
          sleep 1
          sec_count += 1
          break  if sec_count > @sec_limit
        end
        if sec_count > @sec_limit
          @MSHTTP.abort
          return false
        end
        if @MSHTTP.status == 200
          @body = @MSHTTP.responseBody.pack("c*")
        else
          @body = ''
        end
        @MSHTTP.abort
      end
    else
      begin
        @body = open(url).read
      rescue
        return false
      end
    end
    return @body
  end
end
$mshttp = Mshttp.new($exlap_os)


# 数値文字参照(&#……;)を文字に変換
    # 出力文字コードは $BASE_C (j, e, s, w など)に従う。
def ncr_convert(str)
  unless str =~ /\&\#.+;/
    return str
  end
  str = str.gsub(/(\&\#\d+;|\&\#x[0-9a-f]+;)+/i) {|part_str|
    part_str.gsub!(/\&\#x([0-9a-f]+);/i) {|ch|
      n = $1.hex
      ch = "&##{n.to_s};"
    }
    part_str.gsub!(/\&\#(\d+);/) {|ch|
      n = $1.to_i
      if n > 0xff
        ch = [n].pack("n")
      end
    }
    part_str = NKF.nkf("-m0W16#{$BASE_C}", part_str)
  }
  return str
end


# html中のtable部分をrubyの配列に変換。セル結合に対応
    # data_type(配列の各要素のタイプ)  :TEXT | :HTML
    # :HTMLの場合は <td>……</td> が配列の各要素に代入される
    # $cell_count=「空欄以外のセル数」, $merge_count=「セル結合の個数」
def table_to_array(tbl, data_type=:TEXT)
  aa = []
  $cell_count = $merge_count = 0
  if !defined?(Hpricot) and !defined?(Nokogiri)
    STDERR.puts "can't call \"table_to_array\" without hpricot or nokogiri"
    return aa
  end
  if (e = tbl.at("caption"))
    w = e.inner_text.sub(/^\s+/, "").sub(/\s+$/, "")
    w = "caption:" + w
    aa.push([w])
  end
  trs = tbl.search("tr")
  span = {}
  trs.each_with_index {|e, row_n|
    a = []
    col_n = 0
    e.search("*").each {|e2|
      next  unless e2.name =~ /^(th|td)$/i
      while span[[row_n, col_n]]
        a.push(nil)  # セルの結合箇所の埋め草をnilにする
        col_n += 1
      end
      if data_type == :TEXT
        data = e2.inner_text
      else
        data = e2.to_html
      end
      cspan = 1
      if e2['colspan'] =~ /(\d+)/
        cspan = $1.to_i
      end
      rspan = 1
      if e2['rowspan'] =~ /(\d+)/
        rspan = $1.to_i
      end
      if cspan == 1 and rspan == 1  # セル結合とは無関係
        a.push(data)
        $cell_count += 1
      else
        hs = {'data'=>data, 'rspan'=>rspan, 'cspan'=>cspan}
        a.push(hs)
        $cell_count += 1
        $merge_count += 1
        for r in row_n...(row_n+rspan)
          for c in col_n...(col_n+cspan)
            span[[r, c]] = true
          end
        end
      end
      col_n += 1
    }
    aa.push(a)
  }
  return aa
end


# 指定されたurlのwebページを取得し、htmlパーサ用に修正
  # urlが '<' で始まる場合は html source の文字列とみなす
def get_html(url)
  if url =~ /^\s*</
    body = url.sub(/^\s+/, "")
  else
    body = $mshttp.web_open(url)
  end
  if body == '' or body == false
    STDERR.printf("could not access: %s\n", url)
    return ''
  end
  body = NKF.nkf("-m0#{$BASE_C}", body.gsub(/\r\n/, "\n"))
  body = ncr_convert(body)
  body.gsub!(/<pre[^>]*>.+?<\/pre>/im) {|mp|
    mp = mp.gsub(/\n/, "__jfZR6XRr__")
  }
  body.gsub!(/\n+/, "")
  body.gsub!(/<(p|br|li|[oud]l|dt|dd|h[1-6]|div|center|blockquote|pre)([ >])/i,
      "\n<\\1\\2")
  body.gsub!(/<\/(p|li|[oud]l|dt|dd|h[1-6]|div|center|blockquote|pre)>/i,
      "</\\1>\n")
  body.gsub!(/<(hr|hr +[^>]+)>/i, "\n<\\1>\n")
  body.gsub!(/\n\n+/, "\n")
  body.gsub!(/__jfZR6XRr__/, "\n")
  body.gsub!(/<img[^>]+>/i) {|mp|  # 画像をalt注釈に置換
    if mp =~ /alt\s*=\s*["'](.+?)["']/im
      w = $1
      w.sub!(/^\s+/, "")
      w.sub!(/\s+$/, "")
      mp = w
    end
  }
  body.gsub!(/\&nbsp;/i, "&#32;")
  return body
end


# web tableをcsvファイルに書き出す
  # 実際に書き出したtableの個数を返す
def csv_from_web_table(csv_filename, *url_info)
  write_count = 0
  if defined?(Hpricot) or defined?(Nokogiri)
    csv_ary = Array.new
    while url = url_info.shift
      unless url.class == String
        url_info.shift
        next
      end
      table_select_regexp = table_select_count = nil
      while url_info.size > 0 and url_info[0].class != String
        if url_info[0].class == Regexp
          table_select_regexp = url_info.shift
        elsif [Fixnum,Bignum].include?(url_info[0].class)
          table_select_count = url_info.shift
        else
          url_info.shift
        end
      end
      body = get_html(url)
      next  if body == ''
      if defined?(Hpricot)
        doc = Hpricot(body)
      elsif defined?(Nokogiri)
        doc = Nokogiri::HTML.parse(NKF.nkf("-m0w",body), nil, "UTF-8")
      end
      tbl_count = 0
      doc.search("table").each {|tbl_obj|
        tbl_count += 1
        next  if tbl_obj.at("table")  # table内蔵のtableは対象外
        tbl_str = tbl_obj.inner_html  # table部分のhtml source
        if table_select_regexp
          unless table_select_regexp =~ tbl_str
            next
          end
        end
        aa = table_to_array(tbl_obj)
        next  if aa.size < 1
        if table_select_count
          if aa.size < table_select_count
            next
          end
        end
        aa = arrange_array(aa, :TO_STRING, $CSV_C)
        if csv_ary.size > 0
          csv_ary << []
          csv_ary << []
        end
        csv_ary += aa
        write_count += 1
      }
    end
    if write_count > 0
      CSV.open(csv_filename, "w") {|writer|
        csv_ary.each {|row|
          writer << row
        }
      }
    end
  else  # if !defined?(Hpricot) and !defined?(Nokogiri)
    STDERR.puts "can't call \"csv_from_web_table\" " +
        "without hpricot or nokogiri"
  end
  return write_count
end


# web tableをxmlssファイルに書き出す
  # 書き出したワークシートの個数を返す
def xml_from_web_table(xml_filename, *url_info)
  sheet_count = 0
  if defined?(Hpricot) or defined?(Nokogiri)
    aa_ary = Array.new
    while url = url_info.shift
      unless url.class == String
        url_info.shift
        next
      end
      table_select_regexp = table_select_count = nil
      while url_info.size > 0 and url_info[0].class != String
        if url_info[0].class == Regexp
          table_select_regexp = url_info.shift
        elsif [Fixnum,Bignum].include?(url_info[0].class)
          table_select_count = url_info.shift
        else
          url_info.shift
        end
      end
      body = get_html(url)
      next  if body == ''
      if defined?(Hpricot)
        doc = Hpricot(body)
      elsif defined?(Nokogiri)
        doc = Nokogiri::HTML.parse(NKF.nkf("-m0w",body), nil, "UTF-8")
      end
      tbl_count = 0
      doc.search("table").each {|tbl_obj|
        tbl_count += 1
        next  if tbl_obj.at("table")  # table内蔵のtableは対象外
        tbl_str = tbl_obj.inner_html  # table部分のhtml source
        if table_select_regexp
          unless table_select_regexp =~ tbl_str
            next
          end
        end
        aa = table_to_array(tbl_obj)
        next  if aa.size < 1
        if table_select_count
          if aa.size < table_select_count
            next
          end
        end
        aa_ary << aa
        sheet_count += 1
      }
    end
    if sheet_count > 0
      File.open(xml_filename, "w") {|ff|  ff.write make_xml(*aa_ary)}
    end
  else  # if !defined?(Hpricot) and !defined?(Nokogiri)
    STDERR.puts "can't call \"xml_from_web_table\" " +
        "without hpricot or nokogiri"
  end
  return sheet_count
end


# csvをxmlssファイルとして書き出す
  # 書き出したワークシートの個数を返す
def xml_from_csv(xml_filename, *opt)
  aa_ary = Array.new
  sheet_count = 0
  while obj = opt.shift
    if obj.class != String
      opt.shift
      next
    end
    mode = obj.index("\n")  ?  :STRING  :  :FILE
    while opt.size > 0 and opt[0].class == Symbol
      mode = opt.shift
    end
    if mode == :FILE
      filename = obj
      str = File.open(filename) {|ff|  ff.read}
    elsif mode == :STRING
      str = obj
    end
    ary = CSV.parse(NKF.nkf("-m0#{$BASE_C}", str))
    next  if ary.size < 1
    aa_ary << ary
    sheet_count += 1
  end
  if sheet_count > 0
    File.open(xml_filename, "w") {|ff|  ff.write make_xml(*aa_ary)}
  end
  return sheet_count
end
